module Borges

##
# Register a new application that starts with entry_point or at a block

def self.register_application(name, entry_point = nil, &block)
  Application.register(name, entry_point || block)
end

##
#

class Application

  Apps = {}
  
  attr_accessor :entry_point
  attr_reader :sessions

  ##
  # Register an Application

  def self.register(name, entry_point)
    Apps[name] ||= self.new(entry_point)
  end
  
  ##
  # Pass a request off to the appropriate Application

  def self.handle_request(request, response)
    response.application_key = request.application_key
    if app = Apps[request.application_key]
      app.handle_request(request, response)
    else
      response.not_found("No such application: #{request.application_key}",
        request)
    end
  end

  ##
  #

  def initialize(entry_point)
    @sessions = Cache.new
    @entry_point = entry_point
  end
  
  ##
  # Create a new session for this request if the session_key does
  # not exist
  #
  # Otherwise, pass the request to the session and set the response's
  # session_key to the request's session_key

  def handle_request(request, response)
    if request.session_key
      session = @sessions[request.session_key]
      response.session_key = request.session_key
    end
    session ||= new_session(request, response)
    session.handle_request(request, response)
  end
  
  private

  ##
  # Create a new session, and set the response's session_key

  def new_session(request, response)
    session = Session.new(self)
    key = @sessions.store(session)
    response.session_key = key
    session
  end

  def start(&block)
    @entry_point = block
  end
end

end

