class Cache < Hash
  def store(item)
    @count ||= 0
    @count += 1
    self[@count.to_s] = item
    @count.to_s
  end
end

module Borges

##
# Reap sessions that have lived for more than 10 minutes without activity.
# TODO: replace with a hi/low watermark type algorithm, so we reap sessions
# more often when there is high usage.

def self.start_session_reaper
  Thread.start do
    loop do
      sleep(60)
      reap_time = Time.now
      apps = Borges::Application::Apps
      apps.each do |app_name, app|
        app.sessions.delete_if do |id, session|
          if reap_time - session.last_access > 600
            session.expire 
            true
          else
            false
          end
        end
      end
      GC.start
    end
  end
end

##
# Base class for Request

class Request
  include Enumerable

  attr_reader :application_key
  attr_reader :session_key
  attr_reader :action_key

  attr_reader :url
  attr_reader :fields

  def initialize(url, fields)
    @url = url
    @fields = fields

    # application / session_id / action_id
    @url =~ %r!^#{Borges::base_url}(.*?)/?(?:(\d+)(?:/(\d+))?)?$!
    @application_key = $1
    @session_key = $2
    @action_key = $3
  end

  def each
    @fields.each{|a| yield a}
  end

  def [](key)
    @fields[key]
  end
end

class Response

  attr_accessor :body, :headers, :status, :contentType
  attr_accessor :session_key, :action_key, :application_key
  
  def initialize
    @body = ""
    @headers = {}
    @status = 200
    @contentType = "text/html"
  end
  
  def <<(text)
    @body << text
  end
  
  def not_found(msg, request)
    @status = 404
    self << "<h1>#{msg}</h1>"
  end

  def error(msg, request)
    @status = 500
    self << "<h1 style=\"color:red\">#{msg}</h1>"
  end
  
  def action_url(key = @action_key)
    session_url + "/" + key
  end
  
  def session_url
    application_url + "/" + @session_key
  end
  
  def application_url
    Borges.base_url + @application_key
  end
end

##
# Fetch the base url

def self.base_url
  @@base_url
end

##
# Set the base url, must begin and end with a '/'

def self.base_url=(url)
  unless url[0].chr == '/' and url[-1].chr == '/' then
    raise "Invalid base url: ``#{url}'', must begin with a '/'"
  end

  @@base_url = url
end

@@base_url = "/"

end

